/*
	This program (post_get) is developed based on MIRS0401 program get1.c.

	post_get is developed by Shinji Ushimaru.
	get1.c was developed by T.Mano (member of MIRS0401)
	Function: post_get
	MIRS acts as follows in this function.
	1. Turn to 90 degree.
	2. Go arround post with detecting IRS signal.
	3. If find IRS signal, MIRS memorize IRS code number and gets the post.
	4. MIRS retuns to the start position of 1.
*/
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <pthread.h>
#include "func_extern.h"
#include "control.h"
#include "irs_ts_ws.h"
#include "pwm_enc.h"
#include "uss.h"

//DEFINE SENSORS
#define IRS_FRONT	0
#define IRS_LEFT	1

//DEFINE TIME&DIS&ANG
#define TURN_TIME	1300
#define DETECTING_TIME	14000
#define SLEEP_TIME	200000
#define PUSH_TIME	3000
#define IRS_SLEEP1	300000
#define IRS_SLEEP2	700000
#define PWM_SLEEP	20000
#define LOOP_SLEEP	1000

//OTHER DEFINE
#define RETRY_NUMBER	8

int post_get(int target_post){

	int rot_cur=0;
	int retry_num=0;
	int dis2=0;
	int circle_radious2=circle_radious;
	int fwd_dis=0,back_dis=0;
	int rot_time=0;
	int ts_on=0;
	int get_code=0;
	int code_check=0;
	int ir_code=0;
	int uss_1=0,uss_2=0,uss_retry_count=0;
	int irs_on=0;
	int irs_count;

	printf("Start post_get\n\n");
	lqr_rotate(-90,TURN_TIME);
	usleep(SLEEP_TIME);

	//exit(0);

	// Start encoder task with period of 10msec
	printf("Start encoder task.\n");
	pwm_enco_start(10);
	usleep(PWM_SLEEP);

	//󳫻
	printf("Go arround target post\n");
	pwm_lqr_circle(circle_radious2, AROUND_ANG,DETECTING_TIME);

	count.br=0;

	//ֳcheck
	while(1){
		ir_code=irs_code_get(IRS_LEFT);
		if(ir_code!=0){
			usleep(IRS_SLEEP1);
			// current rotation angle arround the post
			rot_cur=90/pi*(alw+arw)/circle_radious2;
			//tire stop
			pwm_both_data(0,0);
			usleep(PWM_SLEEP);
			//stop encoder task
			printf("Stop encoder task.\n");
			pwm_enco_stop();
			usleep(PWM_SLEEP);
			break;
		}

		if(count.br==1) {
			//tire stop
			pwm_both_data(0,0);
			usleep(PWM_SLEEP);
			pwm_enco_stop();
			usleep(PWM_SLEEP);
			printf("\n***********IRS was not detected.***********\n\n");
			goto BY2;
		}
	}

	printf("irs find!\n");
	printf("rot_cur=%d \n",rot_cur);

        while(code_check<5){
                get_code=irs_code_get(1);
                if(get_code!=0) post.code[target_post]=get_code;
                if(get_code==target_code){goto NEXT;}
                usleep(50000);
                //printf("ir_code=%d \n",ir_code);
                code_check++;
        }
        goto BY;
NEXT:

	lqr_rotate(90,TURN_TIME);
	usleep(SLEEP_TIME);

	retry_num=1;
	while(retry_num<RETRY_NUMBER){
		ts_on=0;
		irs_on=0;

		uss_retry_count=0;
		while(uss_retry_count<10){
			do{
				uss_1=uss_get(1);
				uss_2=uss_get(2);
				usleep(LOOP_SLEEP);
			}while (uss_1==0 || uss_2==0);
			if(abs(uss_1-uss_2)< 8){
				break;
			}else if(uss_1>uss_2){
				lqr_rotate(-10,800);
			}else if(uss_1<uss_2){
				lqr_rotate(13,800);
			}
			usleep(LOOP_SLEEP);
			uss_retry_count++;
		}
		usleep(SLEEP_TIME);
		
		printf("\nretry_num=%d\n\n",retry_num);
		
		dis2=(170+30*retry_num);
		
		pwm_enco_start(10);
		usleep(PWM_SLEEP);
		pwm_lqr_straight(dis2,PUSH_TIME);
		usleep(PWM_SLEEP);
		count.br=0;
	
		while(1){
			if(count.br==1){
				break;
			}
			if(ts[0]=='1'|| ts[1]=='1' || ts[2]=='1'){
				usleep(IRS_SLEEP2);
				fwd_dis=(alw+arw)/2;
				ts_on=1;
				break;
			}
			usleep(LOOP_SLEEP);
		}
		//tire stop
		pwm_both_data(0,0);
		usleep(PWM_SLEEP);
		//encoder task stop
		pwm_enco_stop();
		usleep(PWM_SLEEP);
		printf("\nfwd_dis=%d\n",fwd_dis);
		usleep(SLEEP_TIME);
	
		if(fwd_dis!=0){
			back_dis=(-1*fwd_dis);
		}else{
			back_dis=(-1*dis2);
		}

		printf("back_dis=%d\n\n",back_dis);
		lqr_straight(back_dis,back_dis*4);
		usleep(SLEEP_TIME);

		//judgement if post got.
		for(irs_count=0;irs_count<300;irs_count++){
			if(irs[IRS_FRONT]=='1'){
				irs_on=1;
			}
			usleep(LOOP_SLEEP);
		}
		
		if(ts_on==1 && irs_on==0){
			printf("\nGetting post was success!!!\n");
			post.got_number++;
			post.got_code=target_code;
			usleep(SLEEP_TIME);
			break;
		}else if(ts_on!=1){
			printf("\nGetting post was failed.  ts_on==%d\n\n",ts_on);
		}else if(irs_on!=0){
			printf("\nGetting post was failed.  irs_on==%d\n\n",irs_on);
		}

		if(retry_num==RETRY_NUMBER){
				printf("\nretry_num is max. So retry is end.\n");
		}
		retry_num++;
	}
	circle_position(); 

	lqr_rotate(-90,TURN_TIME);

BY:
	printf("move to start_circle\n");
	rot_time = (AROUND_ANG-rot_cur)*27;
	lqr_circle(circle_radious2,AROUND_ANG-rot_cur,rot_time);
BY2:
	usleep(SLEEP_TIME);
	printf("returned to start_circle\n\n");
	printf("finished post_get().\n\n");

	return 0;
}
