/*
	function post_search
	MIRS searches posts during rotation at the center of the field.
	Post positon (its angle and distance) is mesuared by USS.
	Post postion is determined by a sophisticated algorism.
	If number of post found in the algorism is not equal the real
	post number, MIRS try to serach again.
*/

#include <stdio.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <pthread.h>

#include "func_extern.h"
#include "control.h"
#include "irs_ts_ws.h"
#include "pwm_enc.h"
#include "uss.h"

int post_search(int center_count)
{
	int i,j=0;
	int post_no=0;
	int uss_dis_prev2=9998, uss_dis_prev1=9998, uss_dis_cur=9998;

        int theta = 360;
	int angle;
        int left_mm, right_mm;

	int point=0,bef_point=0;
	int ns_count=0;
	int before[30];
	int uss_ang[30];
	int uss_dis[30];
	int uss_bef_point[30];
	int uss_after_point[30];
	int uss_point[30];
	int point_flag[30];
	int after_count[30];
	int post_i[30],post_is[30];
	int uss_ang_prev=0;
	int prev_point=0;
	int is,ii;
	int data_num=7,num2,num3 = 0;
	// tokten means minimum value to recognize that the minimum point
	// of USS distance corresponds to the post center.
	//int tokuten=5;
	int tokuten=0;

	FILE *fp;
	FILE *fps;

	fp=fopen("uss_log.txt","w");
	fps=fopen("angle_se.txt","w");

	for(i=0;i<30;i++){
		before[i]=0;
		uss_ang[i]=0;
		uss_dis[i]=0;
		uss_bef_point[i]=0;
		uss_after_point[i]=0;
		uss_point[i]=0;
		after_count[i]=0;
		point_flag[i]=0;
		post_i[i]=0;
		post_is[i]=0;
	}

        left_mm = theta/180. *pi *diameter / 2.;
        right_mm = left_mm;

        printf("Post search task  start\n");
	
	uss_msg.command=2; /* USS periodic task */
        uss_msg.number = 3; /* USS number to be used */
	uss_msg.period = 100; /* USS task period [msec]*/

	write(ctl, &uss_msg, sizeof(uss_msg));
        printf("uss_task start\n");

	count.br=0;
	pwm_both_data(0,0);
	pwm_enco_start(10);
	pwm_lqr_rotate(360,6500);

	 while(count.left_sum<0){
                usleep(20000);
        }
	
	while(count.br==0) {

		fprintf(fp,"uss=%d left_sum=%d\n",uss_data,count.left_sum);

		if(uss_data<USS_MAX && uss_data>USS_MIN && abs(uss_dis_cur-uss_dis_prev1)<10){

			bef_point=0;
			for(i=0;i<data_num;i++){
				bef_point+=before[i];
			}
			for(i=1;i<data_num;i++){
				before[data_num-i]=before[data_num-i-1];
			}
		fprintf(fp,"bef0=%d bef1=%d bef2=%d bef3=%d bef4=%d bef5=%d\n",before[0],before[1],before[2],before[3],before[4],before[5]);

			uss_dis_cur=uss_data;

			if(uss_dis_prev1>uss_dis_cur){point=1;}
			if(uss_dis_prev1<uss_dis_cur){point=-1;}
			if(uss_dis_prev1==uss_dis_cur){point=0;}
			before[0]=point;

			if(uss_dis_prev2 > uss_dis_prev1 && uss_dis_prev1 < uss_dis_cur){
				uss_dis[post_no]=uss_dis_prev1;
				angle=180*diameter*count.left_sum/(dist_ct*cpr*gear)+90;
				fprintf(fp,"uss_data=%d angle=%d\n",uss_data,angle);
				if(angle>=360) angle-=360;
				uss_ang[post_no]=angle;
				uss_bef_point[post_no]=bef_point;
				point_flag[post_no]=1;
				printf("count.left_sum = %d\n",count.left_sum);
				printf("post_no = %d  uss_dis_cur=%d angel=%d\n",post_no,uss_dis_prev1,angle);
				post_no++;
			}

			for(i=ns_count;i<post_no;i++){
				if(point_flag[i]==1){
					if(uss_dis_prev1<uss_dis_cur){point=1;}
					if(uss_dis_prev1>uss_dis_cur){point=-1;}
					if(uss_dis_prev1==uss_dis_cur){point=0;}
					uss_after_point[i]+=point;
					after_count[i]++;
					if(after_count[i]>=data_num) {point_flag[i]=0;}
				}
			}

		}

		fprintf(fp,"uss_cur=%d,uss_prev1=%d uss_prev2=%d\n",uss_dis_cur,uss_dis_prev1,uss_dis_prev2);

		if(uss_dis_prev1!=uss_dis_cur){
			uss_dis_prev2=uss_dis_prev1;
			uss_dis_prev1=uss_dis_cur;
		}

		usleep(100000);

        }
        pwm_both_data(0, 0);
	usleep(20000);
        pwm_enco_stop();
	usleep(20000);
	fclose(fp);

	// Determin post position 

	for(i=0;i<post_no;i++){
		//uss_point[i]=uss_bef_point[i]+uss_after_point[i];
		uss_point[i]=uss_bef_point[i];
		printf("no=%d bef_point=%d after_point=%d\n",i,uss_bef_point[i],uss_after_point[i]);
	}

	ii=0;
	for(i=0;i<post_no;i++){
		if(uss_point[i]>tokuten){
			post_i[ii]=i;
			ii++;
		}
	}
	num2=ii;

	is=post_i[0];
	post_is[0]=is;
	uss_ang_prev=uss_ang[is];
	prev_point=uss_point[is];
	ii=1;
	for(i=1;i<num2;i++){
		is=post_i[i];
		// If angle difference is less than 20 degree, select 
		// one having higher poits.
		if(abs(uss_ang[is]-uss_ang_prev)<20){
			if(uss_point[is]>prev_point){
				post_is[ii-1]=is;
				prev_point=uss_point[is];
				ii++;
			}
		}else{
			prev_point=0;
			post_is[ii]=is;
			ii++;
		}
		uss_ang_prev=uss_ang[is];
	}
	num3=ii;
		
	if(center_count ==1){
		post.number=num3;
		j = 0;
		//for(i=post.got_number;i<post.number+post.got_number;i++){
		for(i=0;i<post.number;i++){
			is=post_is[j];
			post.ang[i]=uss_ang[is]+POST_ANG_DIFF;
			post.dis[i]=uss_dis[is]+USS_CORRECT+MCENTER_USS3_DIS+post_diameter/2;
       		//	printf("is=%d ang=%d dis=%d\n",is,uss_ang[is],uss_dis[is]);
			fprintf(fps,"searched post angle %d = %d\n",i,post.ang[i]);
			j++;
		}
		fprintf(fps,"post.number in this function = %d\n",post.number);
		fprintf(fps,"post.got_number = %d\n",post.got_number);
		post.number=post.got_number + post.number;
		fprintf(fps,"all post.number = %d\n",post.number);
	}else{
		post_tmp.number=num3;
		for(i=0;i<post_tmp.number;i++){
			is=post_is[i];
			post_tmp.ang[i]=uss_ang[is]+POST_ANG_DIFF;
			post_tmp.dis[i]=uss_dis[is]+USS_CORRECT+MCENTER_USS3_DIS+post_diameter/2;
       			printf("is=%d ang=%d dis=%d\n",is,uss_ang[is],uss_dis[is]);
		}
	}

	uss_msg.command=3; /* USS periodic task STOP*/
	write(ctl, &uss_msg, sizeof(uss_msg));
        usleep(40000);
	
        printf("Post search task stopped\n");
	
	fclose(fps);
	return 0;
}
